### R code from vignette source 'IPMpack_Vignette.Rnw'
### Encoding: ISO8859-1

###################################################
### code chunk number 1: IPMpack_Vignette.Rnw:157-158
###################################################
library(IPMpack)


###################################################
### code chunk number 2: IPMpack_Vignette.Rnw:162-163
###################################################
dff <- generateData()


###################################################
### code chunk number 3: IPMpack_Vignette.Rnw:166-167
###################################################
head(dff)


###################################################
### code chunk number 4: fig0
###################################################
plot(dff$size, dff$sizeNext, xlab = "Size at t", ylab = "Size at t+1")


###################################################
### code chunk number 5: fig0
###################################################
plot(dff$size, dff$sizeNext, xlab = "Size at t", ylab = "Size at t+1")


###################################################
### code chunk number 6: IPMpack_Vignette.Rnw:201-203
###################################################
gr1 <- makeGrowthObj(dataf = dff, Formula = sizeNext~size+size2)
sv1 <- makeSurvObj(dff, Formula = surv~size+size2)


###################################################
### code chunk number 7: IPMpack_Vignette.Rnw:208-209
###################################################
gr1


###################################################
### code chunk number 8: fig1
###################################################
par(mfrow = c(1, 2), bty = "l", pty = "m")
p1 <- picGrow(dff, gr1)
p2 <- picSurv(dff, sv1, ncuts = 30)


###################################################
### code chunk number 9: fig1
###################################################
par(mfrow = c(1, 2), bty = "l", pty = "m")
p1 <- picGrow(dff, gr1)
p2 <- picSurv(dff, sv1, ncuts = 30)


###################################################
### code chunk number 10: IPMpack_Vignette.Rnw:227-231
###################################################
Pmatrix <- createIPMPmatrix(nBigMatrix = 50, 
                            minSize = -5, maxSize = 35, 
                            growObj = gr1, survObj = sv1, 
                            correction = "constant")


###################################################
### code chunk number 11: triDiag
###################################################
diagnosticsPmatrix(Pmatrix, gr1, sv1, dff,correction = "constant")


###################################################
### code chunk number 12: IPMpack_Vignette.Rnw:270-271
###################################################
slotNames(Pmatrix)


###################################################
### code chunk number 13: IPMpack_Vignette.Rnw:274-275
###################################################
Pmatrix@meshpoints


###################################################
### code chunk number 14: fig2
###################################################
par(mfrow=c(1,2),bty="l",pty="s")
persp(Pmatrix@meshpoints, Pmatrix@meshpoints, Pmatrix, ticktype = "detailed", 
		  xlab = "Size at t", ylab = "Size at t+1", zlab="")
image(Pmatrix@meshpoints, Pmatrix@meshpoints, t(Pmatrix), 
		  xlab = "Size at t", ylab = "Size at t+1")
contour(Pmatrix@meshpoints, Pmatrix@meshpoints, t(Pmatrix), add=TRUE)


###################################################
### code chunk number 15: IPMpack_Vignette.Rnw:295-297
###################################################
LE <- meanLifeExpect(Pmatrix)
pTime <- passageTime(mean(dff$size, na.rm = TRUE), Pmatrix)


###################################################
### code chunk number 16: fig3
###################################################
par(mfrow = c(1, 2), bty = "l")
plot(Pmatrix@meshpoints, LE, type = "l", xlab = "Size", 
     ylab = "Mean life expectancy", 
     xlim = range(dff$size, na.rm = TRUE), 
     ylim = range(LE[Pmatrix@meshpoints<max(dff$size, na.rm = TRUE)]))   
plot(Pmatrix@meshpoints, pTime, type = "l", 
     xlab = "Size at start", ylab = "Time to reach chosen size", 
     xlim = range(Pmatrix@meshpoints[Pmatrix@meshpoints<mean(dff$size, na.rm = TRUE)]), 
     ylim = range(pTime[Pmatrix@meshpoints<max(dff$size, na.rm = TRUE)]))
abline(v = mean(dff$size, na.rm = TRUE), col = 2)  #show the target size in red


###################################################
### code chunk number 17: IPMpack_Vignette.Rnw:323-330
###################################################
fv1 <- makeFecObj(dff, Formula = fec~size, 	
                  Family = "gaussian", 
                  Transform = "log")
Fmatrix <- createIPMFmatrix(nBigMatrix = 50, minSize = -5,
                            maxSize = 35, 
                            fecObj = fv1, 
                            correction = "constant")


###################################################
### code chunk number 18: IPMpack_Vignette.Rnw:346-350
###################################################
IPM <- Pmatrix + Fmatrix
Re(eigen(IPM)$value[1])
sensitivity <- sens(IPM)
elasticity <- elas(IPM)


###################################################
### code chunk number 19: fig4
###################################################
par(mfrow = c(1, 3), bty = "l", pty = "s")
plot(Pmatrix@meshpoints, abs(Re(eigen(IPM)$vector[ , 1])), 
		 type = "l", xlab = "Size", 
     ylab = "Stable size structure")
image(Pmatrix@meshpoints, Pmatrix@meshpoints, log(sensitivity), 
		 xlab = "Size at t", 
      ylab = "Size at t+1", main = "Sensitivity")
image(Pmatrix@meshpoints, Pmatrix@meshpoints, log(elasticity), 
		 xlab = "Size at t", 
      ylab = "Size at t+1", main = "Elasticity")


###################################################
### code chunk number 20: IPMpack_Vignette.Rnw:372-375
###################################################
res <- sensParams(growObj = gr1, survObj = sv1, fecObj = fv1, 
                  nBigMatrix = 50, minSize = -5, maxSize = 15)
res


###################################################
### code chunk number 21: fig4a
###################################################
par(mfrow = c(2, 1), bty = "l", pty = "m")
barplot(res$slam, main = expression("Parameter sensitivity of "*lambda), 
		    las = 2, cex.names = 0.5)
barplot(res$elam, main = expression("Parameter elasticity of "*lambda), 
		    las = 2, cex.names = 0.5)


###################################################
### code chunk number 22: fig4a
###################################################
par(mfrow = c(2, 1), bty = "l", pty = "m")
barplot(res$slam, main = expression("Parameter sensitivity of "*lambda), 
		    las = 2, cex.names = 0.5)
barplot(res$elam, main = expression("Parameter elasticity of "*lambda), 
		    las = 2, cex.names = 0.5)


###################################################
### code chunk number 23: IPMpack_Vignette.Rnw:404-405
###################################################
dff <- generateDataDiscrete()


###################################################
### code chunk number 24: IPMpack_Vignette.Rnw:408-409
###################################################
table(dff$stage)


###################################################
### code chunk number 25: IPMpack_Vignette.Rnw:417-422
###################################################
fv1 <- makeFecObj(dataf = dff, Transform = "log", 
                  offspringSplitter = data.frame(continuous = 0.2, 
                  dormant = 0, seedAge1 = 0.8, seedOld = 0), 
                  fecByDiscrete = data.frame(dormant = 0, 
                  seedAge1 = 0, seedOld = 0))


###################################################
### code chunk number 26: IPMpack_Vignette.Rnw:431-435
###################################################
Fmatrix <- createIPMFmatrix(fecObj = fv1, nBigMatrix = 5, 
                            minSize = min(dff$size, na.rm = TRUE), 
                            maxSize = max(dff$size, na.rm = TRUE), 
                            correction = "constant")


###################################################
### code chunk number 27: IPMpack_Vignette.Rnw:439-442
###################################################
gr1 <- makeGrowthObj(dataf = dff, 
                      Formula = sizeNext~size)
sv1 <- makeSurvObj(dff,  Formula = surv~size)


###################################################
### code chunk number 28: IPMpack_Vignette.Rnw:445-446
###################################################
discTrans <- makeDiscreteTrans(dff)


###################################################
### code chunk number 29: IPMpack_Vignette.Rnw:452-457
###################################################
Pmatrix <- createIPMPmatrix(nBigMatrix = 5, 	
                            growObj = makeGrowthObj(dff), 
                            survObj = makeSurvObj(dff), 
                            discreteTrans = discTrans, 
                            correction = "constant")


###################################################
### code chunk number 30: IPMpack_Vignette.Rnw:460-462
###################################################
print(Pmatrix)
print(Fmatrix)


###################################################
### code chunk number 31: IPMpack_Vignette.Rnw:466-467
###################################################
print(Pmatrix+Fmatrix)


###################################################
### code chunk number 32: IPMpack_Vignette.Rnw:481-484
###################################################
dff <- generateData()
env1 <- makeEnvObj(dff) 
env1


###################################################
### code chunk number 33: IPMpack_Vignette.Rnw:488-490
###################################################
gr1 <- makeGrowthObj(dff, Formula = sizeNext~size+covariate)
sv1 <- makeSurvObj(dff, Formula = surv~size+covariate)


###################################################
### code chunk number 34: IPMpack_Vignette.Rnw:502-507
###################################################
Pmatrix <- createCompoundPmatrix(nBigMatrix = 50, minSize = -5, 
                                 maxSize = 35, 
                                 envMatrix = env1, growObj = gr1, 
                                 survObj = sv1, 
                                 correction = "constant")


###################################################
### code chunk number 35: figCompound
###################################################
image(1:nrow(Pmatrix), 1:ncol(Pmatrix), t(log(Pmatrix)), 
	xlab = "Continuous stage (e.g. size) at t", 
		ylab = "Continuous stage (e.g. size) at t+1", axes = FALSE)
axis(1, at = 1:nrow(Pmatrix), lab = round(rep(Pmatrix@meshpoints,Pmatrix@nEnvClass), 2))
axis(2, at = 1:nrow(Pmatrix), lab = round(rep(Pmatrix@meshpoints,Pmatrix@nEnvClass), 2))
abline(h = length(Pmatrix@meshpoints) * (1:Pmatrix@nEnvClass))
abline(v = length(Pmatrix@meshpoints) * (1:Pmatrix@nEnvClass))


###################################################
### code chunk number 36: IPMpack_Vignette.Rnw:531-532
###################################################
pTimes <- stochPassageTime(Pmatrix@meshpoints[15], Pmatrix, env1)


###################################################
### code chunk number 37: fig5
###################################################
par(mfrow = c(1, 1), bty = "l")
#plot(Pmatrix@meshpoints, LEs[1, ], type = "l", xlab = "Size", 
#     ylab = "Mean life expectancy", ylim = c(0, 8), 
#     xlim = range(dff$size, na.rm = TRUE))   
#for (k in 1:Pmatrix@nEnvClass)  {
#	points(Pmatrix@meshpoints, LEs[k, ], type = "l", col = k)
#	}
plot(Pmatrix@meshpoints, pTimes[1:Pmatrix@nBigMatrix], 
     type  = "l", xlab = "Current size", 
     ylab = "Time to reach chosen size", 
     xlim = range(Pmatrix@meshpoints[1:14]), ylim = c(0, 8))
 for (i in 1:Pmatrix@nEnvClass) {
	points(Pmatrix@meshpoints, 
               pTimes[((i-1) * Pmatrix@nBigMatrix+1):(Pmatrix@nBigMatrix*i)], 
               type = "l", col = i)
  }


###################################################
### code chunk number 38: IPMpack_Vignette.Rnw:564-568
###################################################
IPMlist <- makeListIPMs(dataf = dff, nBigMatrix = 25, minSize = -5, 
                        maxSize = 35, explSurv = surv~size+covariate, 
                        explGrow = sizeNext~size+size2+covariate, 
                        explFec = fec~size, Transform="log",correction = "constant")


###################################################
### code chunk number 39: IPMpack_Vignette.Rnw:577-579
###################################################
stochGrowthRateSampleList(listIPMmatrix = IPMlist, 
                          nRunIn = 30, tMax = 50)


###################################################
### code chunk number 40: IPMpack_Vignette.Rnw:593-600
###################################################
dff <- generateDataStoch()
sv1 <- makeSurvObj(dataf = dff, 
                          Formula = surv~size+covariate1+covariate3)
gr1 <- makeGrowthObj(dataf = dff, 
                             Formula = sizeNext~size+covariate1+covariate2)
fv1 <- makeFecObj(dataf = dff, fecConstants = data.frame(1.8), 
                  Formula = fec~size, Transform = "log")


###################################################
### code chunk number 41: IPMpack_Vignette.Rnw:603-604
###################################################
head(dff)


###################################################
### code chunk number 42: IPMpack_Vignette.Rnw:607-608
###################################################
gr1


###################################################
### code chunk number 43: IPMpack_Vignette.Rnw:618-623
###################################################
tVals <- seq(1, 4, by = 1/12)
covTest <- c(1 + 0.5*sin(2*pi*tVals))
covMatTest <- data.frame(covariate1 = rnorm(length(covTest), covTest, 0.5) - 1, 
                         covariate2 = rnorm(length(covTest), covTest, 0.5) - 1, 
                         covariate3 = rnorm(length(covTest), covTest, 0.5) - 1)


###################################################
### code chunk number 44: IPMpack_Vignette.Rnw:627-634
###################################################
r <- stochGrowthRateManyCov(covariate = covMatTest, nRunIn = 12*1, 
                            tMax = length(tVals), growthObj = gr1, 
                            survObj = sv1, fecObj = fv1, nBigMatrix = 20, 
                            minSize = 2*min(dff$size, na.rm = TRUE), 
                            maxSize = 1.6*max(dff$size, na.rm = TRUE), 
                            nMicrosites = 50, correction = "constant")
print(r)


###################################################
### code chunk number 45: IPMpack_Vignette.Rnw:649-656
###################################################
dff <- generateData()
grlist <- makePostGrowthObjs(dff, 
                             explanatoryVariables = "size", 
                             burnin = 100, nitt = 400)
svlist <- makePostSurvivalObjs(dff, 
                               explanatoryVariables = "size", 
                               burnin = 100, nitt = 400)


###################################################
### code chunk number 46: IPMpack_Vignette.Rnw:661-665
###################################################
PmatrixList <- makeListPmatrix(grlist, svlist, nBigMatrix = 20, 
                               minSize = -5, 
                               maxSize = 35, 
                               correction = "constant")


###################################################
### code chunk number 47: IPMpack_Vignette.Rnw:668-670
###################################################
res <- getIPMoutput(PmatrixList, targetSize = 5, FmatrixList = NULL)
names(res)


###################################################
### code chunk number 48: IPMpack_Vignette.Rnw:675-681
###################################################
fv <- makePostFecObjs(dff, explanatoryVariables = "size+size2", fecConstants=data.frame(0.01), 
                      burnin = 100, nitt = 400, Transform = "log")
FmatrixList <- makeListFmatrix(fv, nBigMatrix = 20, minSize = -5, 
                               maxSize = 35, cov = FALSE,
                               correction = "constant")
res <- getIPMoutput(PmatrixList, targetSize = 5, FmatrixList)


###################################################
### code chunk number 49: fig7
###################################################
par(mfrow = c(2, 2), bty = "l")
nMesh <- PmatrixList[[1]]@nBigMatrix
CI <- apply(res$LE, 2, quantile, c(0.025, 0.975))

plot(PmatrixList[[1]]@meshpoints, colMeans(res$LE), type = "n", 
     xlab = "Size", 
     ylab = "Life expectancy", 
     ylim = range(quantile(res$LE, c(0.025, 0.975), na.rm = TRUE)), 
     xlim = range(dff$size, na.rm = TRUE))
     
polygon(c(PmatrixList[[1]]@meshpoints)[c(1:nMesh, nMesh:1)], 
        c(CI[1, 1:nMesh], CI[2, nMesh:1]), col = "grey", border = "white")
points(PmatrixList[[1]]@meshpoints, colMeans(res$LE), type = "l", lty = 3)

CI <- apply(res$pTime, 2, quantile, c(0.025, 0.975))

plot(PmatrixList[[1]]@meshpoints, colMeans(res$pTime), 
     type = "n", xlab = "Size", ylab = "Passage time", 
     xlim = range(dff$size[dff$size<5], na.rm = TRUE), 
     ylim = range(quantile(
     res$pTime[, PmatrixList[[1]]@meshpoints<max(dff$size, na.rm = TRUE)], 
				 c(0.025, 0.975), na.rm = TRUE)))

polygon(c(PmatrixList[[1]]@meshpoints)[c(1:nMesh, nMesh:1)], 
        c(CI[1, 1:nMesh], CI[2, nMesh:1]), col = "grey", border = "white")
points(PmatrixList[[1]]@meshpoints, colMeans(res$pTime), type = "l", lty = 3)

res$stableStage <- abs(Re(res$stableStage))
res$stableStage <- res$stableStage/rowSums(res$stableStage)
res$stableStage[!is.finite(res$stableStage) | is.na(res$stableStage)] <- 0
CI <- apply(res$stableStage, 2, quantile, c(0.025, 0.975))

plot(PmatrixList[[1]]@meshpoints, colMeans(res$stableStage), type = "n", 
		 xlab = "Size", ylab = "Stable.size", 
     ylim = range(quantile(res$stableStage, c(0.025, 0.975), na.rm = TRUE)))
polygon(c(PmatrixList[[1]]@meshpoints)[c(1:nMesh, nMesh:1)], 
        c(CI[1, 1:nMesh], CI[2, nMesh:1]), col = "grey", border = "white")
points(PmatrixList[[1]]@meshpoints, colMeans(res$stableStage, na.rm = TRUE), 
       type = "l", lty = 3)

hist(res$lambda, xlab = expression(lambda), ylab = "", main = "")


###################################################
### code chunk number 50: IPMpack_Vignette.Rnw:748-749
###################################################
setClass("growthObjSaturate", representation(paras = "numeric", sd = "numeric"))


###################################################
### code chunk number 51: IPMpack_Vignette.Rnw:752-757
###################################################
fSaturate <- function(size, pars) { 
    u <- exp(pmin(pars[1] + pars[2] * size, 50))
    u <- pars[3] * 1/(1+u)
    return(u)
}


###################################################
### code chunk number 52: IPMpack_Vignette.Rnw:762-769
###################################################
wrapSaturate <- function(par, dataf) { 
    pred <- fSaturate(dataf$size, par[1:3])
    ss <- sum((pred - dataf$sizeNext)^2, na.rm = TRUE)
    return(ss)
    }
tmp <- optim(c(1, 1, 1), wrapSaturate, dataf = dff, method = "Nelder-Mead")
tmp    


###################################################
### code chunk number 53: IPMpack_Vignette.Rnw:774-776
###################################################
resids <- fSaturate(dff$size, tmp$par) - dff$sizeNext
sdSaturate <- sd(resids, na.rm = TRUE)


###################################################
### code chunk number 54: IPMpack_Vignette.Rnw:781-784
###################################################
gr1 <- new("growthObjSaturate")
gr1@paras <- tmp$par
gr1@sd <- sdSaturate


###################################################
### code chunk number 55: IPMpack_Vignette.Rnw:789-796
###################################################
setMethod("growth", c("numeric", "numeric", "numeric", "growthObjSaturate"), 
          function(size, sizeNext, cov, growthObj){
              mux <- fSaturate(size, growthObj@paras)
              sigmax <- growthObj@sd
              u <- dnorm(sizeNext, mux, sigmax, log = F)  
              return(u);
          })


